use crate::ui::UIWidget;
use maud::{Markup, Render, html};

#[allow(non_snake_case)]
pub fn FitWidth<T: UIWidget + 'static>(inner: T) -> FitWidthWidget {
    FitWidthWidget(Box::new(inner))
}

pub struct FitWidthWidget(Box<dyn UIWidget>);

impl Render for FitWidthWidget {
    fn render(&self) -> Markup {
        self.render_with_class("")
    }
}

impl UIWidget for FitWidthWidget {
    fn can_inherit(&self) -> bool {
        true
    }

    fn base_class(&self) -> Vec<String> {
        vec!["max-w-fit".to_string()]
    }

    fn extended_class(&self) -> Vec<String> {
        let mut c = self.base_class();
        c.extend_from_slice(&self.0.extended_class());
        c
    }

    fn render_with_class(&self, class: &str) -> Markup {
        if self.0.as_ref().can_inherit() {
            self.0
                .as_ref()
                .render_with_class(&format!("{} {class}", self.base_class().join(" ")))
        } else {
            html! {
                div class=(format!("{} {class}", self.base_class().join(" "))) {
                    (self.0.as_ref())
                }
            }
        }
    }
}
